@@ -85,6 +85,7 @@ from synapse.http.replicationagent import ReplicationAgent
 from synapse.http.types import QueryParams
 from synapse.logging.context import make_deferred_yieldable, run_in_background
 from synapse.logging.opentracing import set_tag, start_active_span, tags
+from synapse.metrics import SERVER_NAME_LABEL
 from synapse.types import ISynapseReactor, StrSequence
 from synapse.util import json_decoder
 from synapse.util.async_helpers import timeout_deferred
@@ -108,9 +109,13 @@ except ImportError:
 
 logger = logging.getLogger(__name__)
 
-outgoing_requests_counter = Counter("synapse_http_client_requests", "", ["method"])
+outgoing_requests_counter = Counter(
+    "synapse_http_client_requests", "", labelnames=["method", SERVER_NAME_LABEL]
+)
 incoming_responses_counter = Counter(
-    "synapse_http_client_responses", "", ["method", "code"]
+    "synapse_http_client_responses",
+    "",
+    labelnames=["method", "code", SERVER_NAME_LABEL],
 )
 
 # the type of the headers map, to be passed to the t.w.h.Headers.
@@ -213,7 +218,7 @@ class _IPBlockingResolver:
 
                 if _is_ip_blocked(ip_address, self._ip_allowlist, self._ip_blocklist):
                     logger.info(
-                        "Blocked %s from DNS resolution to %s" % (ip_address, hostname)
+                        "Blocked %s from DNS resolution to %s", ip_address, hostname
                     )
                     has_bad_ip = True
 
@@ -318,7 +323,7 @@ class BlocklistingAgentWrapper(Agent):
             pass
         else:
             if _is_ip_blocked(ip_address, self._ip_allowlist, self._ip_blocklist):
-                logger.info("Blocking access to %s" % (ip_address,))
+                logger.info("Blocking access to %s", ip_address)
                 e = SynapseError(HTTPStatus.FORBIDDEN, "IP address blocked")
                 return defer.fail(Failure(e))
 
@@ -346,6 +351,7 @@ class BaseHttpClient:
         treq_args: Optional[Dict[str, Any]] = None,
     ):
         self.hs = hs
+        self.server_name = hs.hostname
         self.reactor = hs.get_reactor()
 
         self._extra_treq_args = treq_args or {}
@@ -384,7 +390,9 @@ class BaseHttpClient:
             RequestTimedOutError if the request times out before the headers are read
 
         """
-        outgoing_requests_counter.labels(method).inc()
+        outgoing_requests_counter.labels(
+            method=method, **{SERVER_NAME_LABEL: self.server_name}
+        ).inc()
 
         # log request but strip `access_token` (AS requests for example include this)
         logger.debug("Sending request %s %s", method, redact_uri(uri))
@@ -438,7 +446,11 @@ class BaseHttpClient:
 
                 response = await make_deferred_yieldable(request_deferred)
 
-                incoming_responses_counter.labels(method, response.code).inc()
+                incoming_responses_counter.labels(
+                    method=method,
+                    code=response.code,
+                    **{SERVER_NAME_LABEL: self.server_name},
+                ).inc()
                 logger.info(
                     "Received response to %s %s: %s",
                     method,
@@ -447,7 +459,11 @@ class BaseHttpClient:
                 )
                 return response
             except Exception as e:
-                incoming_responses_counter.labels(method, "ERR").inc()
+                incoming_responses_counter.labels(
+                    method=method,
+                    code="ERR",
+                    **{SERVER_NAME_LABEL: self.server_name},
+                ).inc()
                 logger.info(
                     "Error sending request to  %s %s: %s %s",
                     method,
@@ -723,7 +739,7 @@ class BaseHttpClient:
         resp_headers = dict(response.headers.getAllRawHeaders())
 
         if response.code > 299:
-            logger.warning("Got %d when downloading %s" % (response.code, url))
+            logger.warning("Got %d when downloading %s", response.code, url)
             raise SynapseError(
                 HTTPStatus.BAD_GATEWAY, "Got error %d" % (response.code,), Codes.UNKNOWN
             )
@@ -821,12 +837,12 @@ class SimpleHttpClient(BaseHttpClient):
         pool.cachedConnectionTimeout = 2 * 60
 
         self.agent: IAgent = ProxyAgent(
-            self.reactor,
-            hs.get_reactor(),
+            reactor=self.reactor,
+            proxy_reactor=hs.get_reactor(),
             connectTimeout=15,
             contextFactory=self.hs.get_http_client_context_factory(),
             pool=pool,
-            use_proxy=use_proxy,
+            proxy_config=hs.config.server.proxy_config,
         )
 
         if self._ip_blocklist:
@@ -855,6 +871,7 @@ class ReplicationClient(BaseHttpClient):
             hs: The HomeServer instance to pass in
         """
         super().__init__(hs)
+        self.server_name = hs.hostname
 
         # Use a pool, but a very small one.
         pool = HTTPConnectionPool(self.reactor)
@@ -891,7 +908,9 @@ class ReplicationClient(BaseHttpClient):
             RequestTimedOutError if the request times out before the headers are read
 
         """
-        outgoing_requests_counter.labels(method).inc()
+        outgoing_requests_counter.labels(
+            method=method, **{SERVER_NAME_LABEL: self.server_name}
+        ).inc()
 
         logger.debug("Sending request %s %s", method, uri)
 
@@ -948,7 +967,11 @@ class ReplicationClient(BaseHttpClient):
 
                 response = await make_deferred_yieldable(request_deferred)
 
-                incoming_responses_counter.labels(method, response.code).inc()
+                incoming_responses_counter.labels(
+                    method=method,
+                    code=response.code,
+                    **{SERVER_NAME_LABEL: self.server_name},
+                ).inc()
                 logger.info(
                     "Received response to %s %s: %s",
                     method,
@@ -957,7 +980,11 @@ class ReplicationClient(BaseHttpClient):
                 )
                 return response
             except Exception as e:
-                incoming_responses_counter.labels(method, "ERR").inc()
+                incoming_responses_counter.labels(
+                    method=method,
+                    code="ERR",
+                    **{SERVER_NAME_LABEL: self.server_name},
+                ).inc()
                 logger.info(
                     "Error sending request to  %s %s: %s %s",
                     method,
@@ -1106,7 +1133,7 @@ class _MultipartParserProtocol(protocol.Protocol):
                         self.stream.write(data[start:end])
                     except Exception as e:
                         logger.warning(
-                            f"Exception encountered writing file data to stream: {e}"
+                            "Exception encountered writing file data to stream: %s", e
                         )
                         self.deferred.errback()
                     self.file_length += end - start
@@ -1129,7 +1156,7 @@ class _MultipartParserProtocol(protocol.Protocol):
         try:
             self.parser.write(incoming_data)
         except Exception as e:
-            logger.warning(f"Exception writing to multipart parser: {e}")
+            logger.warning("Exception writing to multipart parser: %s", e)
             self.deferred.errback()
             return
 
