@@ -110,6 +110,7 @@ def _load_rules(
         msc3381_polls_enabled=experimental_config.msc3381_polls_enabled,
         msc4028_push_encrypted_events=experimental_config.msc4028_push_encrypted_events,
         msc4210_enabled=experimental_config.msc4210_enabled,
+        msc4306_enabled=experimental_config.msc4306_enabled,
     )
 
     return filtered_rules
@@ -146,6 +147,7 @@ class PushRulesWorkerStore(
             db=database,
             notifier=hs.get_replication_notifier(),
             stream_name="push_rules_stream",
+            server_name=self.server_name,
             instance_name=self._instance_name,
             tables=[
                 ("push_rules_stream", "instance_name", "stream_id"),
@@ -163,8 +165,9 @@ class PushRulesWorkerStore(
         )
 
         self.push_rules_stream_cache = StreamChangeCache(
-            "PushRulesStreamChangeCache",
-            push_rules_id,
+            name="PushRulesStreamChangeCache",
+            server_name=self.server_name,
+            current_stream_pos=push_rules_id,
             prefilled_cache=push_rules_prefill,
         )
 
