@@ -169,9 +169,9 @@ class RelationsWorkerStore(SQLBaseStore):
     @cached(uncached_args=("event",), tree=True)
     async def get_relations_for_event(
         self,
+        room_id: str,
         event_id: str,
         event: EventBase,
-        room_id: str,
         relation_type: Optional[str] = None,
         event_type: Optional[str] = None,
         limit: int = 5,
@@ -324,7 +324,7 @@ class RelationsWorkerStore(SQLBaseStore):
                         account_data_key=0,
                         push_rules_key=0,
                         to_device_key=0,
-                        device_list_key=0,
+                        device_list_key=MultiWriterStreamToken(stream=0),
                         groups_key=0,
                         un_partial_stated_rooms_key=0,
                     )
