@@ -184,11 +184,19 @@ class MultiWriterIdGenerator(AbstractStreamIdGenerator):
 
     Note: Only works with Postgres.
 
+    Warning: Streams using this generator start at ID 2, because ID 1 is always assumed
+        to have been 'seen as persisted'.
+        Unclear if this extant behaviour is desirable for some reason.
+        When creating a new sequence for a new stream,
+        it will be necessary to use `START WITH 2`.
+
     Args:
         db_conn
         db
         stream_name: A name for the stream, for use in the `stream_positions`
             table. (Does not need to be the same as the replication stream name)
+        server_name: The homeserver name of the server (used to label metrics)
+            (this should be `hs.hostname`).
         instance_name: The name of this instance.
         tables: List of tables associated with the stream. Tuple of table
             name, column name that stores the writer's instance name, and
@@ -204,10 +212,12 @@ class MultiWriterIdGenerator(AbstractStreamIdGenerator):
 
     def __init__(
         self,
+        *,
         db_conn: LoggingDatabaseConnection,
         db: DatabasePool,
         notifier: "ReplicationNotifier",
         stream_name: str,
+        server_name: str,
         instance_name: str,
         tables: List[Tuple[str, str, str]],
         sequence_name: str,
@@ -217,6 +227,7 @@ class MultiWriterIdGenerator(AbstractStreamIdGenerator):
         self._db = db
         self._notifier = notifier
         self._stream_name = stream_name
+        self.server_name = server_name
         self._instance_name = instance_name
         self._positive = positive
         self._writers = writers
@@ -269,6 +280,9 @@ class MultiWriterIdGenerator(AbstractStreamIdGenerator):
         self._known_persisted_positions: List[int] = []
 
         # The maximum stream ID that we have seen been allocated across any writer.
+        # Since this defaults to 1, this means that ID 1 is assumed to have already
+        # been 'seen'. In other words, multi-writer streams start at 2.
+        # Unclear if this is desirable behaviour.
         self._max_seen_allocated_stream_id = 1
 
         # The maximum position of the local instance. This can be higher than
@@ -552,6 +566,7 @@ class MultiWriterIdGenerator(AbstractStreamIdGenerator):
             txn.call_after(
                 run_as_background_process,
                 "MultiWriterIdGenerator._update_table",
+                self.server_name,
                 self._db.runInteraction,
                 "MultiWriterIdGenerator._update_table",
                 self._update_stream_positions_table_txn,
@@ -588,6 +603,7 @@ class MultiWriterIdGenerator(AbstractStreamIdGenerator):
             txn.call_after(
                 run_as_background_process,
                 "MultiWriterIdGenerator._update_table",
+                self.server_name,
                 self._db.runInteraction,
                 "MultiWriterIdGenerator._update_table",
                 self._update_stream_positions_table_txn,
@@ -794,20 +810,16 @@ class MultiWriterIdGenerator(AbstractStreamIdGenerator):
 
         # We upsert the value, ensuring on conflict that we always increase the
         # value (or decrease if stream goes backwards).
-        if isinstance(self._db.engine, PostgresEngine):
-            agg = "GREATEST" if self._positive else "LEAST"
-        else:
-            agg = "MAX" if self._positive else "MIN"
+        cmp = "<" if self._positive else ">"
 
-        sql = """
+        sql = f"""
             INSERT INTO stream_positions (stream_name, instance_name, stream_id)
             VALUES (?, ?, ?)
             ON CONFLICT (stream_name, instance_name)
             DO UPDATE SET
-                stream_id = %(agg)s(stream_positions.stream_id, EXCLUDED.stream_id)
-        """ % {
-            "agg": agg,
-        }
+                stream_id = EXCLUDED.stream_id
+                WHERE stream_positions.stream_id {cmp} EXCLUDED.stream_id
+        """
 
         pos = self.get_current_token_for_writer(self._instance_name)
         txn.execute(sql, (self._stream_name, self._instance_name, pos))
