@@ -45,11 +45,13 @@ from typing import (
     overload,
 )
 
-from twisted.internet import reactor
+from twisted.internet import defer, reactor
 from twisted.internet.interfaces import IReactorTime
 
 from synapse.config import cache as cache_config
-from synapse.metrics.background_process_metrics import wrap_as_background_process
+from synapse.metrics.background_process_metrics import (
+    run_as_background_process,
+)
 from synapse.metrics.jemalloc import get_jemalloc_stats
 from synapse.util import Clock, caches
 from synapse.util.caches import CacheMetric, EvictionReason, register_cache
@@ -118,103 +120,121 @@ USE_GLOBAL_LIST = False
 GLOBAL_ROOT = ListNode["_Node"].create_root_node()
 
 
-@wrap_as_background_process("LruCache._expire_old_entries")
-async def _expire_old_entries(
-    clock: Clock, expiry_seconds: float, autotune_config: Optional[dict]
-) -> None:
+def _expire_old_entries(
+    server_name: str,
+    clock: Clock,
+    expiry_seconds: float,
+    autotune_config: Optional[dict],
+) -> "defer.Deferred[None]":
     """Walks the global cache list to find cache entries that haven't been
     accessed in the given number of seconds, or if a given memory threshold has been breached.
     """
-    if autotune_config:
-        max_cache_memory_usage = autotune_config["max_cache_memory_usage"]
-        target_cache_memory_usage = autotune_config["target_cache_memory_usage"]
-        min_cache_ttl = autotune_config["min_cache_ttl"] / 1000
 
-    now = int(clock.time())
-    node = GLOBAL_ROOT.prev_node
-    assert node is not None
+    async def _internal_expire_old_entries(
+        clock: Clock, expiry_seconds: float, autotune_config: Optional[dict]
+    ) -> None:
+        if autotune_config:
+            max_cache_memory_usage = autotune_config["max_cache_memory_usage"]
+            target_cache_memory_usage = autotune_config["target_cache_memory_usage"]
+            min_cache_ttl = autotune_config["min_cache_ttl"] / 1000
 
-    i = 0
+        now = int(clock.time())
+        node = GLOBAL_ROOT.prev_node
+        assert node is not None
 
-    logger.debug("Searching for stale caches")
+        i = 0
 
-    evicting_due_to_memory = False
+        logger.debug("Searching for stale caches")
 
-    # determine if we're evicting due to memory
-    jemalloc_interface = get_jemalloc_stats()
-    if jemalloc_interface and autotune_config:
-        try:
-            jemalloc_interface.refresh_stats()
-            mem_usage = jemalloc_interface.get_stat("allocated")
-            if mem_usage > max_cache_memory_usage:
-                logger.info("Begin memory-based cache eviction.")
-                evicting_due_to_memory = True
-        except Exception:
-            logger.warning(
-                "Unable to read allocated memory, skipping memory-based cache eviction."
-            )
+        evicting_due_to_memory = False
 
-    while node is not GLOBAL_ROOT:
-        # Only the root node isn't a `_TimedListNode`.
-        assert isinstance(node, _TimedListNode)
-
-        # if node has not aged past expiry_seconds and we are not evicting due to memory usage, there's
-        # nothing to do here
-        if (
-            node.last_access_ts_secs > now - expiry_seconds
-            and not evicting_due_to_memory
-        ):
-            break
-
-        # if entry is newer than min_cache_entry_ttl then do not evict and don't evict anything newer
-        if evicting_due_to_memory and now - node.last_access_ts_secs < min_cache_ttl:
-            break
-
-        cache_entry = node.get_cache_entry()
-        next_node = node.prev_node
-
-        # The node should always have a reference to a cache entry and a valid
-        # `prev_node`, as we only drop them when we remove the node from the
-        # list.
-        assert next_node is not None
-        assert cache_entry is not None
-        cache_entry.drop_from_cache()
-
-        # Check mem allocation periodically if we are evicting a bunch of caches
-        if jemalloc_interface and evicting_due_to_memory and (i + 1) % 100 == 0:
+        # determine if we're evicting due to memory
+        jemalloc_interface = get_jemalloc_stats()
+        if jemalloc_interface and autotune_config:
             try:
                 jemalloc_interface.refresh_stats()
                 mem_usage = jemalloc_interface.get_stat("allocated")
-                if mem_usage < target_cache_memory_usage:
-                    evicting_due_to_memory = False
-                    logger.info("Stop memory-based cache eviction.")
+                if mem_usage > max_cache_memory_usage:
+                    logger.info("Begin memory-based cache eviction.")
+                    evicting_due_to_memory = True
             except Exception:
                 logger.warning(
-                    "Unable to read allocated memory, this may affect memory-based cache eviction."
+                    "Unable to read allocated memory, skipping memory-based cache eviction."
                 )
-                # If we've failed to read the current memory usage then we
-                # should stop trying to evict based on memory usage
-                evicting_due_to_memory = False
 
-        # If we do lots of work at once we yield to allow other stuff to happen.
-        if (i + 1) % 10000 == 0:
-            logger.debug("Waiting during drop")
-            if node.last_access_ts_secs > now - expiry_seconds:
-                await clock.sleep(0.5)
-            else:
-                await clock.sleep(0)
-            logger.debug("Waking during drop")
+        while node is not GLOBAL_ROOT:
+            # Only the root node isn't a `_TimedListNode`.
+            assert isinstance(node, _TimedListNode)
 
-        node = next_node
+            # if node has not aged past expiry_seconds and we are not evicting due to memory usage, there's
+            # nothing to do here
+            if (
+                node.last_access_ts_secs > now - expiry_seconds
+                and not evicting_due_to_memory
+            ):
+                break
 
-        # If we've yielded then our current node may have been evicted, so we
-        # need to check that its still valid.
-        if node.prev_node is None:
-            break
+            # if entry is newer than min_cache_entry_ttl then do not evict and don't evict anything newer
+            if (
+                evicting_due_to_memory
+                and now - node.last_access_ts_secs < min_cache_ttl
+            ):
+                break
 
-        i += 1
+            cache_entry = node.get_cache_entry()
+            next_node = node.prev_node
 
-    logger.info("Dropped %d items from caches", i)
+            # The node should always have a reference to a cache entry and a valid
+            # `prev_node`, as we only drop them when we remove the node from the
+            # list.
+            assert next_node is not None
+            assert cache_entry is not None
+            cache_entry.drop_from_cache()
+
+            # Check mem allocation periodically if we are evicting a bunch of caches
+            if jemalloc_interface and evicting_due_to_memory and (i + 1) % 100 == 0:
+                try:
+                    jemalloc_interface.refresh_stats()
+                    mem_usage = jemalloc_interface.get_stat("allocated")
+                    if mem_usage < target_cache_memory_usage:
+                        evicting_due_to_memory = False
+                        logger.info("Stop memory-based cache eviction.")
+                except Exception:
+                    logger.warning(
+                        "Unable to read allocated memory, this may affect memory-based cache eviction."
+                    )
+                    # If we've failed to read the current memory usage then we
+                    # should stop trying to evict based on memory usage
+                    evicting_due_to_memory = False
+
+            # If we do lots of work at once we yield to allow other stuff to happen.
+            if (i + 1) % 10000 == 0:
+                logger.debug("Waiting during drop")
+                if node.last_access_ts_secs > now - expiry_seconds:
+                    await clock.sleep(0.5)
+                else:
+                    await clock.sleep(0)
+                logger.debug("Waking during drop")
+
+            node = next_node
+
+            # If we've yielded then our current node may have been evicted, so we
+            # need to check that its still valid.
+            if node.prev_node is None:
+                break
+
+            i += 1
+
+        logger.info("Dropped %d items from caches", i)
+
+    return run_as_background_process(
+        "LruCache._expire_old_entries",
+        server_name,
+        _internal_expire_old_entries,
+        clock,
+        expiry_seconds,
+        autotune_config,
+    )
 
 
 def setup_expire_lru_cache_entries(hs: "HomeServer") -> None:
@@ -234,10 +254,12 @@ def setup_expire_lru_cache_entries(hs: "HomeServer") -> None:
     global USE_GLOBAL_LIST
     USE_GLOBAL_LIST = True
 
+    server_name = hs.hostname
     clock = hs.get_clock()
     clock.looping_call(
         _expire_old_entries,
         30 * 1000,
+        server_name,
         clock,
         expiry_time,
         hs.config.caches.cache_autotuning,
@@ -376,9 +398,43 @@ class LruCache(Generic[KT, VT]):
     If cache_type=TreeCache, all keys must be tuples.
     """
 
+    @overload
     def __init__(
         self,
+        *,
         max_size: int,
+        server_name: str,
+        cache_name: str,
+        cache_type: Type[Union[dict, TreeCache]] = dict,
+        size_callback: Optional[Callable[[VT], int]] = None,
+        metrics_collection_callback: Optional[Callable[[], None]] = None,
+        apply_cache_factor_from_config: bool = True,
+        clock: Optional[Clock] = None,
+        prune_unread_entries: bool = True,
+        extra_index_cb: Optional[Callable[[KT, VT], KT]] = None,
+    ): ...
+
+    @overload
+    def __init__(
+        self,
+        *,
+        max_size: int,
+        server_name: Literal[None] = None,
+        cache_name: Literal[None] = None,
+        cache_type: Type[Union[dict, TreeCache]] = dict,
+        size_callback: Optional[Callable[[VT], int]] = None,
+        metrics_collection_callback: Optional[Callable[[], None]] = None,
+        apply_cache_factor_from_config: bool = True,
+        clock: Optional[Clock] = None,
+        prune_unread_entries: bool = True,
+        extra_index_cb: Optional[Callable[[KT, VT], KT]] = None,
+    ): ...
+
+    def __init__(
+        self,
+        *,
+        max_size: int,
+        server_name: Optional[str] = None,
         cache_name: Optional[str] = None,
         cache_type: Type[Union[dict, TreeCache]] = dict,
         size_callback: Optional[Callable[[VT], int]] = None,
@@ -392,8 +448,13 @@ class LruCache(Generic[KT, VT]):
         Args:
             max_size: The maximum amount of entries the cache can hold
 
-            cache_name: The name of this cache, for the prometheus metrics. If unset,
-                no metrics will be reported on this cache.
+            server_name: The homeserver name that this cache is associated with
+                (used to label the metric) (`hs.hostname`). Must be set if `cache_name` is
+                set. If unset, no metrics will be reported on this cache.
+
+            cache_name: The name of this cache, for the prometheus metrics. Must be set
+                if `server_name` is set. If unset, no metrics will be reported on this
+                cache.
 
             cache_type:
                 type of underlying cache to be used. Typically one of dict
@@ -457,11 +518,12 @@ class LruCache(Generic[KT, VT]):
         # do yet when we get resized.
         self._on_resize: Optional[Callable[[], None]] = None
 
-        if cache_name is not None:
+        if cache_name is not None and server_name is not None:
             metrics: Optional[CacheMetric] = register_cache(
-                "lru_cache",
-                cache_name,
-                self,
+                cache_type="lru_cache",
+                cache_name=cache_name,
+                cache=self,
+                server_name=server_name,
                 collect_callback=metrics_collection_callback,
             )
         else:
